part of nueli.core.config;

ShadowXConfig get $shadowX => ShadowXConfig();

class ShadowXConfig extends Config<ShadowXConfig, ShadowX> {
  List<ShadowXConfig> _children = const [];
  Color? _color;
  Offset? _offset;
  double? _spread;
  double? _blur;

  ShadowXConfig() : super(ShadowX());

  ShadowXConfig _config({
    Color? color,
    Offset? offset,
    double? spread,
    double? blur,
  }) {
    _color = color ?? _color;
    _offset = offset ?? _offset;
    _spread = spread ?? _spread;
    _blur = blur ?? _blur;
    return this;
  }

  ShadowXConfig color(Color color) => _config(color: color);
  ShadowXConfig grow(double x, [double? y]) => _config(offset: Offset(x, y ?? x));
  ShadowXConfig blur(double blur) => _config(blur: blur); // ignore: non_constant_identifier_names
  ShadowXConfig spread(double spread) => _config(spread: spread); // ignore: non_constant_identifier_names
  ShadowXConfig glow(double glow) => _config(offset: Offset.zero, blur: glow);

  ShadowXConfig elevate(double elevation) => _config(blur: elevation, spread: -0.5, offset: Offset(elevation / 8, elevation / 8));
  ShadowXConfig raise(double float) => _config(blur: float / 6, spread: -0.5, offset: Offset(float, float));

  ShadowXConfig withPosition(DecorationPosition? position) => this & position;

  @override
  ShadowXConfig operator *(double factor) {
    final shadowXAp = _ShadowXApprox(this);
    _children.map((e) => e * factor).toList();
    return _config(
      color: shadowXAp.color * factor,
      offset: shadowXAp.offset * factor,
      spread: shadowXAp.spread * factor,
      blur: shadowXAp.blur * factor,
    );
  }

  @override
  ShadowXConfig operator -(ShadowXConfig operand) {
    final shadowXAp = _ShadowXApprox(this);
    final shadowXAp2 = _ShadowXApprox(operand);
    _children.map((e) => e - operand).toList();
    return _config(
      color: shadowXAp.color - shadowXAp2.color,
      offset: shadowXAp.offset - shadowXAp2.offset,
      spread: shadowXAp.spread - shadowXAp2.spread,
      blur: shadowXAp.blur - shadowXAp2.blur,
    );
  }

  @override
  ShadowXConfig operator +(ShadowXConfig operand) {
    final shadowXAp = _ShadowXApprox(this);
    final shadowXAp2 = _ShadowXApprox(operand);
    _children.map((e) => e + operand).toList();
    return _config(
      color: shadowXAp.color + shadowXAp2.color,
      offset: shadowXAp.offset + shadowXAp2.offset,
      spread: shadowXAp.spread + shadowXAp2.spread,
      blur: shadowXAp.blur + shadowXAp2.blur,
    );
  }

  @override
  ShadowXConfig lerp(shadow) {
    if (shadow is ShadowXConfig)
      _config(
        color: shadow._color ?? _color,
        blur: shadow._blur ?? _blur,
        spread: shadow._spread ?? _spread,
        offset: shadow._offset ?? _offset,
      );
    return super.lerp(shadow);
  }

  BoxShadow get toBoxShadow {
    final shadowXAp = _ShadowXApprox(this);
    return BoxShadow(
      color: shadowXAp.color,
      offset: shadowXAp.offset,
      blurRadius: shadowXAp.blur,
      spreadRadius: shadowXAp.spread,
    );
  }

  List<BoxShadow> get toBoxShadowDeep => _children.map((e) => e.toBoxShadow).toList()..add(this.toBoxShadow);

  ShadowXConfig save() => ShadowXConfig().._children.addAll([...this._children, this.._children = []]);

  Widget call() {
    return DecoratedBox(
      decoration: BoxDecoration(boxShadow: toBoxShadowDeep),
      child: data.child,
      position: data.position ?? DecorationPosition.background,
    );
  }
}

class ShadowX extends ConfigData<ShadowX> {
  DecorationPosition? position;

  ShadowX({Widget? child, this.position}) : super(child);

  ShadowX lerp(data) {
    if (data is DecorationPosition) position = data;

    if (data is ShadowX) position = data.position;
    return super.lerp(data);
  }
}

class ShadowXTween extends Tween<ShadowXConfig> {
  ShadowXTween({ShadowXConfig? begin, ShadowXConfig? end}) : super(begin: begin, end: end);

  @override
  ShadowXConfig transform(double value) => (begin ?? $shadowX) + ((end ?? $shadowX) - (begin ?? $shadowX)) * value;
}

class ShadowXTransition extends AnimatedWidget {
  final Widget? child;
  ShadowXTransition({required Animation<ShadowXConfig> shadowX, this.child}) : super(listenable: shadowX);
  Animation<ShadowXConfig> get shadowX => listenable as Animation<ShadowXConfig>;

  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
      animation: listenable,
      builder: (c, child) => shadowX.value.withChild(child)(),
    );
  }
}

class _ShadowXApprox {
  final ShadowXConfig shadow;

  _ShadowXApprox(this.shadow);
  Color get color => shadow._color ?? palette.none;
  Offset get offset => shadow._offset ?? Offset.zero;
  double get spread => shadow._spread ?? 0;
  double get blur => shadow._blur ?? 0;
}
